//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon-Suk Lee (jl)
//  FILE: PCGivenStmt.C
//  PROJ: COKO
//  DATE: 10/9/96                                                              
//
// SYNOPSIS
// Class definitions for CGivenStmt.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCGIVENSTMT_C_
#define _PCGIVENSTMT_C_

#include "PCGivenStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CGivenStmt
// Parameters:	(CStmt* a ... )
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CGivenStmt::CGivenStmt(CStmt* a ... )
{
  va_list ap;
  va_start (ap,a);
  _stmt = a;
  _count = 0;
  for(;;)
	{
	  CEqn* p = va_arg(ap,CEqn*);
	  if (p == NULL) 
		break;
	  else 
		_eqnList[_count++] = p;
	}
  _count--;
  va_end(ap);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CGivenStmt
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CGivenStmt::~CGivenStmt(void)
{
  for (int i=0; i <= _count; i++)
	if (_eqnList[i]) { delete _eqnList[i]; _eqnList[i] = NULL; }
  if (_stmt) { delete _stmt; _stmt = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
// First evaluate all equations.
// If all eqns match was successful, execute the body statement. 
// If there was a failure during match eqn then undo all changes made.
//
//////////////////////////////////////////////////////////////////////////////

CState* CGivenStmt::Exec(CState * s)
{
  if (_debug_)
	cout << "-- GivenStmt Begins" << endl;
  
  // Save the pattern we were using and update it with the new one
  // specified in this statement. "CurrentPattern"
  // will be used to build modified kola query right after returning
  // from rule invokes.
  CPattern *old_pattern = s->CurrentPattern();

  s->INew(); // Clear our the top of identifier stack by pushing an empty
             // element.


  s->SetSuccess(T);

  int i = 0;
  int j = 0;
  while ((s->ISuccess() == T) && (i <= _count))
	{
	  s->INew(); 
	  // put new stack frame for each equation
	  s = (_eqnList[i])->Exec(s);
	  i++;
	  j++;
  	}
  i--; 

  if (s->ISuccess()) 
	// If all eqns match was successful, execute the body statement.
	{
	  s = _stmt->Exec(s);
	}
  else
	// If there was a failure during match eqn then undo all changes made.
	{
	  while (i >= 0)
		{
		  if (_debug_)
			{
			  cout << "@PCGiven:EXEC" << endl;
			  cout << "eqn match failed - start undoDEP" << endl;
			}
		  s = (_eqnList[i])->undoDEP(s);
		  i--;
		}
	}

  // clear up stack frames that were created for each equation
  while (j) 
	{
	  s->IOld();
	  (_eqnList[j - 1])->endOfEval();

	  // endOfEval added.
	  // It solves a bug that occured when I used Given stmt and TD 
	  // or BU together.
	  // A stmt inside of TD or BU is evaluated more than one
	  // time on different kola trees and not setting back an equations
	  // _evalFlag causes an error.

	  j--;
	}

  s->IOld(); // Restore the old identifier environment.
  s->SetCurrentPattern(old_pattern); // Restore the old pattern.
  s->CCleanInvDep();
  
  if (_debug_)
	cout << "-- GivenStmt Ends" << endl;

  return s;  
}

#endif

