#include "PAParser.h"

PARuleBlock::PARuleBlock(string name, PACode include, PACode body, PACode pbody)
{
  _name = name;
  _includeCode = include;
  _bodyCode = body;
  _prologCode = pbody;
}

const char *PARuleBlock::Name(void)
{
  return _name.chars();
}

void PARuleBlock::Emit(ofstream & foutH, ofstream & foutC, ofstream & foutSLC, ofstream & foutMainC, ofstream & foutMainH, ofstream & foutPL, int c_type)
{
  PACode::iterator i;

  if (c_type == 2)
	{
	  // Emit directives + class declaration -> Goes into .H
	  foutH << "#ifndef _CORB_" << _name << "_H_" << endl;
	  foutH << "#define _CORB_" << _name << "_H_" << endl;
	  foutH << "#include \"COKO.H\"" << endl;
	  foutH << "#include \"KOFVar.H\"" << endl;
	  foutH << "#include \"KOPVar.H\"" << endl;
	  foutH << "#include \"KOOVar.H\"" << endl;
	  foutH << "#include \"KOBVar.H\"" << endl;

	  foutH << endl;
	  foutH << "class CORB" << _name << " : public CRuleBlock " << endl;
	  foutH << "{" << endl;
	  foutH << "public:" << endl;
	  foutH << "\tCState *Exec(CState *);" << endl;
	  foutH << "};" << endl;
	  foutH << "#endif" << endl;

	  // Emit Exec Body -> Goes into .cc
	  foutC << endl;
	  foutC << "#include \"" << _name << ".h\"" << endl;
	  for(i = _includeCode.begin(); i != _includeCode.end(); i++) {
		foutC << *i << endl;
	  }
	  foutC << endl;
	  foutC << "CState * CORB" << _name << "::Exec(CState *s)" << endl;
	  foutC << "{" << endl;
	  for(i = _bodyCode.begin(); i != _bodyCode.end(); i++) {
		foutC << *i << endl;
	  }
	  foutC << "}" << endl;
	  
	  if(strcmp(_name.chars(), "Main") == 0) {
		foutC << endl;
		foutC << "#include <stdio.h>" << endl;
		foutC << "#include <stdlib.h>" << endl;
		foutC << "#include <iostream.h>" << endl;
		foutC << "#include \"PAParser.H\"" << endl;
		foutC << "#include \"QLParser.H\"" << endl;
		foutC << "#include \"KONonType.H\"" << endl << endl;
		foutC << "int main(int argc, char *argv[])" << endl;
		foutC << "{" << endl;
		foutC << "\tNonType * qTree = NULL;" << endl;
		foutC << "\tif(argc < 1) qTree = PAParser().parse();" << endl;
		foutC << "\telse if(argc == 2) {" << endl;

		foutC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
		foutC << "\t\t\tqTree = PAParser().parse();" << endl;
		foutC << "\telse" << endl;

		foutC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0)" << endl;
		foutC << "\t\t\t_debug_ = 1, qTree = PAParser().parse();" << endl;
		foutC << "\t\telse qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t}" << endl;
		foutC << "\telse if (argc == 3) {" << endl;

		foutC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
		foutC << "\t\t\tqTree = QLParser(argv[2]).parse();" << endl;
		foutC << "\t\telse if(strcmp(\"-q\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\tqTree = QLParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse" << endl;
		
		foutC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0) {" << endl;
		foutC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-ddebug\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t}" << endl;
		
		foutC << "\tif(!qTree) exit(1);" << endl;
		
		foutC << "\tCRuleBlock * MainRB = new CORB" << _name << ";" << endl;
		foutC << "\tCState *s = MainRB->Exec(new CState(qTree,new CIEnvStack(),0));" 
			  << endl;
		// foutC << "\tchar *result = s->Store()->representation();" << endl;
		// foutC << "\tcout << result << endl;" << endl;
		foutC << "\tdelete MainRB; delete s; " << endl;
		// delete [] result;" << endl;
		foutC << "}" << endl;
	  }

	  for(i = _prologCode.begin(); i != _prologCode.end(); i++) {
		foutPL << *i << endl;
	  }
	}
  
  if (c_type == 1)
	{  
	  // added by jl on July 23 1997
	  foutSLC <<  "#include \"" << _name << ".h\"" << endl;
	  foutSLC << endl << endl <<  "extern \"C\" {" << endl;
	  foutSLC << "\tNonType* callOpt (NonType* arg) { " << endl;
	  foutSLC << "\t\tCRuleBlock* tmp = new CORB" << _name <<"();" << endl;
	  foutSLC << "\t\tCState* s = new CState(arg,new CIEnvStack(), 0);" << endl;
	  foutSLC << "\t\ts = tmp->Exec(s);" << endl;
	  //foutSLC << "\t\tNonType* return_val = s->Store();" << endl;
	  foutSLC << "\t\tNonType* return_val = s->Store()->Copy_kTree();" << endl;
	  // foutSLC << "\t\t/* char* result = s->Store()->representation(); */" << endl;
	  // foutSLC << "\t\t/* cout << result << endl; */" << endl;
	  foutSLC << "\t\tdelete tmp; delete s;" << endl;
	  // /* delete [] result; */" << endl;
	  foutSLC << "\t\treturn return_val;" << endl;
	  foutSLC << "\t\t}" << endl;
	  foutSLC << "\t}" << endl << endl;
	  // added by jl on July 23 1997
	}

  if (c_type == 0)
	{
	  foutMainC << "#include \"" << _name << "_main.h\"" << endl;
	  foutMainC << "#include <" << _name << ".h>" << endl;
	  foutMainC << endl;
	  foutMainC << "CState * CORB" << _name << "_main::Exec(CState *s)" << endl;
	  foutMainC << "{" << endl;
	  foutMainC << "CRuleBlock * " << _name << "= new CORB" << _name << ";" <<endl;
	  foutMainC << "CStmt * _rStmt_ =" << endl;
	  foutMainC << "new CRuleInvokeStmt(" << _name << ", NULL);" << endl;
	  foutMainC << "CState * new_s = _rStmt_->Exec(s);" << endl;
	  foutMainC << "delete _rStmt_;" << endl << "delete " << _name << ";" << endl;
	  foutMainC << "return new_s;" << endl << "}" << endl;


	  foutMainH << "#ifndef _CORB_" << _name << "_main_H_" << endl;
	  foutMainH << "#define _CORB_" << _name << "_main_H_" << endl;
	  foutMainH << "#include \"COKO.H\"" << endl;
	  foutMainH << "#include \"KOFVar.H\"" << endl;
	  foutMainH << "#include \"KOPVar.H\"" << endl;
	  foutMainH << "#include \"KOOVar.H\"" << endl;
	  foutMainH << "#include \"KOBVar.H\"" << endl;	  
	  foutMainH << endl;
	  foutMainH << "class CORB" << _name << "_main : public CRuleBlock" << endl;
	  foutMainH << "{" << endl;
	  foutMainH << "public:" << endl;
	  foutMainH << "\tCState *Exec(CState *);" << endl;
	  foutMainH << "};" << endl << "#endif" << endl;


	  foutMainC << endl;
	  foutMainC << "#include <stdio.h>" << endl;
	  foutMainC << "#include <stdlib.h>" << endl;
	  foutMainC << "#include <iostream.h>" << endl;
	  foutMainC << "#include \"PAParser.H\"" << endl;
	  foutMainC << "#include \"KONonType.H\"" << endl << endl;
	  foutMainC << "#include \"QLParser.H\"" << endl;
	  foutMainC << "int main(int argc, char *argv[])" << endl;
	  foutMainC << "{" << endl;
	  foutMainC << "\tNonType * qTree = NULL;" << endl;
	  foutMainC << "\tif(argc < 1) qTree = PAParser().parse();" << endl;
	  foutMainC << "\telse if(argc == 2) {" << endl;

	  foutMainC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\tqTree = PAParser().parse();" << endl;
	  foutMainC << "\telse" << endl;

	  foutMainC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\t_debug_ = 1, qTree = PAParser().parse();" << endl;
	  foutMainC << "\t\telse qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t}" << endl;
	  foutMainC << "\telse if (argc == 3) {" << endl;

	  foutMainC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\tqTree = QLParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-q\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\tqTree = QLParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse" << endl;

	  foutMainC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0) {" << endl;
	  foutMainC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-ddebug\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t}" << endl;
	  foutMainC << "\tif(!qTree) exit(1);" << endl;
	  foutMainC << "\tCRuleBlock * MainRB = new CORB" << _name << "_main;" << endl;
	  foutMainC << "\tCState *s = MainRB->Exec(new CState(qTree,new CIEnvStack(),0));" << endl; 
	  // foutMainC << "\tchar *result = s->Store()->representation();" << endl;
	  // foutMainC << "\tcout << result << endl;" << endl;
	  foutMainC << "\tdelete MainRB; delete s;" << endl;
	  // delete [] result;" << endl;
	  foutMainC << "}" << endl;
	}
}
