/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)
//  FILE: KOBEqu.H 
//  PROJ: COKO Compiler, KOLA Parser
//  DATE: 5/97
//
// SYNOPSIS
//  Equality Comparison Boolean former.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_BEQU_HEADER
#define KOLA_BEQU_HEADER 

#include "KGlobals.H"
#include "KOBool.H"
#include "KOObject.H"
#include "KONameUnion.H"
#include "KONonType.H"

#include "PCState.H"
#include "KOOVar.H"

#ifdef _OSTORE_
#include "OSBPrimitive.H"
#endif // _OSTORE_

#include <string.h>

class BEqu : public koBool
{
public:
  BEqu (koObject* o1,koObject* o2, TBool * t = NULL);
  //  Constructor.

  BEqu (const BEqu& other);
  //  Copy Constructor. 
  
  ~BEqu ();
  //  Destructor. 

  koBool *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 2;}
  //  Number of arguments.

  koObject* One () const {return one;}
  //  What is the Object that is the first argument?
  
  koObject* Two () const {return two;}
  //  What is the Object that is the second argument?
  
  //  BResult* reduce ();
  //  Return 
  //    new BResINF (one -> reduce (), "=", two -> reduce ())
    
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  return "BFO =:" + one.representation + "," + two.representation

  const char * Name() const { return namestring; }
  // Return a character string representing the name of the function.

  void SetOne(NonType* a) {delete one; one = (koObject*)a;}
  // replace existing child to new one.
 
  void SetTwo(NonType* a) {delete two; two = (koObject*)a;}
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // reset currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.

  NonType* transform(CState* s);
  //  Return coresponding NonType object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  char *sql ();
  // Return the SQL string equivalent of this object

  char *oql ();
  // Return the OQL string equivalent of this object

  koBool *boo ();
  // Symbolic simplification of this.  Eventually,
  // this method will replace "bool" presently
  // only defined with _OSTORE_ flag set

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
  // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koBool* bool() { return new BEqu(One()->object(), Two()->object()); }
  // Returns the koBool equivalent of the object.

  OSBool* result();
  // Returns the OSBool equivalent of the object.
#endif // _OSTORE_

protected:
  koObject* one;
  //  What is the Object 1st argument?
  
  koObject* two;
  //  What is the second Object argument?

  int currChild;
  
};
# endif





    




