/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOInt.c
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOInt.H"
extern "C" {int atoi (const char *s);}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OInt
// Parameters:	(char *v)							  
//
// Constructor
//         								    
//////////////////////////////////////////////////////////////////////////////

OInt::OInt (char *v, TObj * t) : koObject (t,v)
{
  val = atoi(v);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OInt
// Parameters:	(int v)							  
//
// Constructor
//         								    
//////////////////////////////////////////////////////////////////////////////

OInt::OInt (int v, TObj * t) : koObject (v, t)
{
  val = v;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OInt
// Parameters:	(Int v)							  
//
// Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OInt::OInt (const OInt& other) : koObject ((koObject &) other)
{
  val = other . Val ();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OInt
// Parameters:	()							  
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OInt::~OInt ()
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koObject*
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject* OInt::Copy () const
{
    return new OInt (*this);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int OInt::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_INT))
  {
    delete name_;
    if (val == (((OInt*)other)->Val()))
      return T;
    else
      return F;
  }
  else
  {
     delete name_;
     return F;
  }
}
 
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OInt::Myname()
{
        NameUnion* a = new NameUnion(ONAME, OBJ_INT);
        return a;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return as_string (val)
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OInt::reduce ()
// {
//     return new OResINT (val);
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return val as a string
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OInt::representation ()
{
  char *res = new char [MAXINTLENGTH];  
  // 32(MAXINTLENGTH) bytes should be enough!
  sprintf (res, "%d", val);
  return res;	
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OInt::transform(CState* s)
{
 
    return new OInt((char*)val,Type());
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int OInt::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_INT) && (((OInt*)nt)->Val() == val))
	{
	  delete tmp_name;
	  return T;
	}
  else 
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OInt::replaceChild(NonType* child, int which_child)
{
}
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OInt::prolog ()
{
  char *res = new char [MAXINTLENGTH + strlen("oint()") + 1];  
  // 32(MAXINTLENGTH) bytes should be enough!
  sprintf (res, "oint(%d)", val);
  return res;	
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OInt::sp_prolog ()
{
  char* text = new char [MAXINTLENGTH + 1]; 
  // 32(MAXINTLENGTH) bytes should be enough!
  sprintf (text, "%d", val);
  
  SP_term_ref nam = SP_new_term_ref();
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(nam,text);
  
  SP_cons_functor(res, SP_atom_from_string("oint"),1,nam);
  
  if (text) { delete text; text = NULL; }
  
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OInt::sql ()
{
  char *res = new char [MAXINTLENGTH + 1];
  sprintf (res, "%d", val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OInt::oql ()
{
  char *res = new char [MAXINTLENGTH + 1];
  sprintf (res, "%d", val);
  return res;
}
