/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOOInvoke.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  The object invoking former.  Takes a function (f)
//  and an object (e) and denotes the object that
//  would be returned when f is invoked on e (f ! e)
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_OINVOKE_HEADER
#define KOLA_OINVOKE_HEADER 

#include "KGlobals.H"
#include "KOFunction.H"
#include "KOObject.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOFUser.H"

#include "PCState.H"
#include "KOOVar.H"
#include "KOFVar.H"

#include <string.h>
#include <iostream.h>

class OInvoke : public koObject
{
public:
  OInvoke (Function* f, koObject* o, TObj * t = NULL);
  //  Constructor.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
 
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  OInvoke (const OInvoke& other);
  //  Copy Constructor. 
  
  ~OInvoke ();
  //  Destructor. 

  koObject *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 2; }
  //  Number of arguments.

  Function* One () const {return one;}
  //  What is the function that is the first argument?
  
  koObject* Two () const {return two;}
  //  What is the object that is the second argument?
    
  //  OResult* reduce ();
  //  Return 
  //    (one -> reduce (two -> reduce ()))
	
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  return "OFO INVOKE:" + one.representation + "," + two.representation

  const char * Name() const { return namestring; }

  void SetOne(NonType* a) { delete one; one = (Function*)a; }
  // replace existing child to new one.
 
  void SetTwo(NonType* a) { delete two; two = (koObject*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  char *oql ();
  // Return the OQL string equivalent of this object

  char *sql ();
  // Return the SQL string equivalent of this object

  koObject *obj ();
  // Symbolic simplification of this.  Eventually,
  // this method will replace "object" presently
  // only defined with _OSTORE_ flag set

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject *object() { return One()->invoke(Two()->object()); }
  // Returns the kola equivalent of an object.

  OSObject *result() { return One()->exec(Two()->result()); }
  // Returns the OSObject equivalent of itself.
#endif // _OSTORE_

protected:
  Function* one;
  //  What is the function that is the first argument?
  
  koObject* two;
  //  What is the Object that is the 2nd argument to
  //  the former?
  
  int currChild;
};


# endif

