/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOLink.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOLink.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OLink
// Parameters:	(koBool* b1, koBool* b2, TBool* t)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OLink::OLink (koObject* o)
{
  _obj = o;
  next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OLink
// Parameters:	(const OLink&)
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OLink::OLink (const OLink & other)
{
  _obj = (other . Obj ()) -> Copy ();
  if (other . Next () == NULL)
    next = NULL;
  else
    next = new OLink (*(other . Next ()));
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OLink
// Parameters:	(koBool* b1, koBool* b2)							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OLink::~OLink ()
{
    delete _obj;
    delete next;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation 
// Parameters:	()
//  return ob -> representation ()
//
/////////////////////////////////////////////////////////////////////////////

char* OLink::representation ()
{
    char *or = _obj -> representation ();
    char *res = new char [strlen (or) + 1];
    sprintf (res, "%s", or);
    delete [] or;
    return res;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OLink::prolog ()
{
  char *or = _obj -> prolog ();
  char *res = new char [strlen (or) + 1];
  sprintf (res, "%s", or);
  delete [] or;
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OLink::sp_prolog ()
{
  return _obj->sp_prolog();
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OLink::sql ()
{
  char *tmp = _obj -> sql ();
  if ((strncmp (tmp, "ERROR", 5) == 0) || next == NULL)
	return tmp;
  else {
	char *n = next -> sql ();
	if (strncmp (n, "ERROR", 5) == 0) {
	  delete [] tmp;
	  return n;
	} else {
	  char *res = new char [strlen (", ") + strlen (tmp) + strlen (n) + 1];
	  sprintf (res, "%s, %s", tmp, n);
	  delete [] tmp;
	  delete [] n;
	  return res;
	}
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OLink::oql ()
{
  char *tmp = _obj -> oql ();
  if ((strncmp (tmp, "ERROR", 5) == 0) || next == NULL)
	return tmp;
  else {
	char *n = next -> oql ();
	if (strncmp (n, "ERROR", 5) == 0) {
	  delete [] tmp;
	  return n;
	} else {
	  char *res = new char [strlen (", ") + strlen (tmp) + strlen (n) + 1];
	  sprintf (res, "%s, %s", tmp, n);
	  delete [] tmp;
	  delete [] n;
	  return res;
	}
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

ObjList* OLink::objlist (ObjList *ol)
{
  ObjList *tmp = ol -> insert (_obj -> obj ());
  if (next == NULL)
	return tmp;
  else
	return next -> objlist (tmp);
}
