/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOTimes.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOTimes.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OTimes
// Parameters:	(koObject* o1, koObject *o2)							  
//
//  Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

OTimes::OTimes (koObject *o1, koObject *o2, TObj * t) : koObject (t,"x")
{
  one = o1;
  two = o2;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OTimes
// Parameters:	(OTimes& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

OTimes::OTimes (const OTimes& other) : koObject ((koObject &) other)
{
  one = (other . One ()) -> Copy ();
  two = (other . Two ()) -> Copy ();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OTimes
// Parameters:	()							  
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

OTimes::~OTimes ()
{
  delete one;
  delete two;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()							  
//
//  Virtual Copy Constructor 
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject *OTimes::Copy () const
{
  return new OTimes (*this);
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////
 
int OTimes::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_TIMES))
  {
    delete name_;
    return ((one->deepeq(((OTimes*)other)->One())) 
         && (two->deepeq(((OTimes*)other)->Two())));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OTimes::Myname()
{
        NameUnion* a = new NameUnion(ONAME, OBJ_TIMES);
        return a;
}
 
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  nextChild
// Parameters:  ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////

NonType* OTimes::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else if (currChild == 1)
	{
	  currChild++;
	  return two;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return new OResINF 
//    (one -> reduce (), " \\I",  two -> reduce ())
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OTimes::reduce ()
// {
//   return new OResINF (one -> reduce (), "\\TIMES", two -> reduce ());
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FFO: "TIMES: " + one -> representation() + ", " + two -> representation() 
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OTimes::representation ()
{
  char *or = one -> representation ();
  char *tr = two -> representation ();
  char *res = new char [strlen (or) + strlen (tr) + strlen ("( TIMES )") + 1];
  sprintf (res, "(%s TIMES %s)", or, tr);
  delete [] or;    
  delete [] tr;    
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OTimes::transform(CState* s)
{
  koObject* first = (koObject*) One() -> transform(s);
  koObject* second = (koObject*) Two() -> transform(s);
  return new OTimes(first,second,Type());
    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transformDeep
// Parameters:	(CState*, CIEnvStackDataType*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.
//
//////////////////////////////////////////////////////////////////////////////

NonType* OTimes::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  koObject* first = (koObject*) One() -> transformDeep(s, startingPt);
  koObject* second = (koObject*) Two() -> transformDeep(s, startingPt);
  return new OTimes(first,second,Type());
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between NonType and NonType.
// Stores a variable and CIdent pair onto the stack.
//
//////////////////////////////////////////////////////////////////////////////

int OTimes::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_TIMES))
	{
	  // do it for first child.
	  if (one->compares(((OTimes*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one)) 
			// child (one) is a variable.
			{
			  if ((a->IFind(((OVar*)one)->Name())) == NULL)
				{ 
				  // child is a new variable.
				  a->IInsert(((OVar*)one)->Name(), new CIdent((OTimes*)nt,((OTimes*)nt)->One(),1));
				  // Push entry into the stack.  
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's 
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((OVar*)one)->Name()))->Child_Look_Up())->deepeq(((OTimes*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++; 
	  // compare failed   
	 
	  // do it for second child.
	  if (two->compares(((OTimes*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((OVar*)two)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)two)->Name(), new CIdent((OTimes*)nt, ((OTimes*)nt)->Two(), 2));
				}
			  else 
				{
				  if (!(((a->IFind(((OVar*)two)->Name()))->Child_Look_Up())->deepeq(((OTimes*)nt)->Two())))
					return_val++;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0. 
  // In this case, return false(0).
  // Otherwise, return true(1).

  delete tmp_name;

  if (return_val)
	return F;
  else
	return T;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This match method is used for matching statement.
// No duplicated variable entry is allowed.
//
//////////////////////////////////////////////////////////////////////////////

int OTimes::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_TIMES))
	{
	  // do it for first child.
	  if (one->comparesSTM(((OTimes*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one)) 
			// child (one) is a variable.
			{
			  if ((a->IFind(((OVar*)one)->Name())) == NULL)
				{ 
				  // child is a new variable.
				  a->IInsert(((OVar*)one)->Name(), new CIdent((OTimes*)nt,((OTimes*)nt)->One(),1));
				  // Push entry into the stack.  
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((OVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++; 
	  // compare failed   
	 
	  // do it for second child.
	  if (two->comparesSTM(((OTimes*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((OVar*)two)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)two)->Name(), new CIdent((OTimes*)nt, ((OTimes*)nt)->Two(), 2));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((OVar*)two)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0. 
  // In this case, return false(0).
  // Otherwise, return true(1).

  delete tmp_name;

  if (return_val)
	return F;
  else
	return T;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list and inverse dependency list
//
//////////////////////////////////////////////////////////////////////////////

void OTimes::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
  two->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on inverse dependency list.
// undo for dependency list is not necessary, since 
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void OTimes::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
  two->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OTimes::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
  else if ( which_child == 2 )
	SetTwo(child);
}


#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: result
//  Parameters:
//
//  Implements the set multiplication koObject
//
//////////////////////////////////////////////////////////////////////////////

OSObject* OTimes::result()
{
  return new CPSieve((Iterator*)One()->result(),
                     (Iterator*)Two()->result());
}
#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OTimes::prolog ()
{
  char *or = one -> prolog ();
  char *tr = two -> prolog ();
  char *res = new char [strlen (or) + strlen (tr) + strlen ("otimes(, )") + 1];
  sprintf (res, "otimes(%s, %s)", or, tr);
  delete [] or;    
  delete [] tr;    
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OTimes::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref tr = two -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("otimes"),2,or,tr);
  return res;
}
#endif // _PROLOG_


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OTimes::sql ()
{
  char *arg1 = one -> sql ();
  char *arg2 = two -> sql ();

  if (strncmp (arg1, "ERROR", 5) == 0) {
	delete [] arg2;
	return arg1;
  } else if (strncmp (arg2, "ERROR", 5) == 0) {
	delete [] arg1;
	return arg2;
  } else {
	OVar *x = new OVar (0);
	OVar *y = new OVar (0);
	char *res = new char [strlen ("(select ,  from  ,  )") + strlen (arg1) + strlen (arg2) + 
						  strlen (x -> Name ()) + strlen (y -> Name ()) + 1];
	sprintf (res, "(select %s, %s from %s %s, %s %s)", x -> Name (), y -> Name (), arg1, arg2);
	delete [] arg1;
	delete [] arg2;
	delete x;
	delete y;
	return res;
  }

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OTimes::oql ()
{
  char *arg1 = one -> oql ();
  char *arg2 = two -> oql ();

  if (strncmp (arg1, "ERROR", 5) == 0) {
	delete [] arg2;
	return arg1;
  } else if (strncmp (arg2, "ERROR", 5) == 0) {
	delete [] arg1;
	return arg2;
  } else {
	OVar *x = new OVar (0);
	OVar *y = new OVar (0);
	char *res = new char [strlen ("(select struct (one: , two: ) from  ,  )") + strlen (arg1) + strlen (arg2) + 
						  strlen (x -> Name ()) + strlen (y -> Name ()) + 1];
	sprintf (res, "(select struct (one: %s, two: %s) from %s %s, %s %s)", x -> Name (), y -> Name (), arg1, arg2);
	delete [] arg1;
	delete [] arg2;
	delete x;
	delete y;
	return res;
  }

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

koObject* OTimes::obj ()
{
  return new OTimes (one -> obj (), two -> obj ());
}

