/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)
//  FILE: COOVar.H 
//  PROJ: COKO
//  DATE: 4/96		                                                    
//
// SYNOPSIS
//  Object associated with variables.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_OVAR_HEADER
#define KOLA_OVAR_HEADER 

#include "PCState.H"
#include "KONameUnion.H"
#include "KGlobals.H"
#include "KOObject.H"
#include <string.h>

class OVar : public koObject
{
    
 public:

  OVar (char* s, TObj * t = NULL);
  //  Constructor.
  
  OVar (const OVar& other);
  //  Copy Constructor.
  
  OVar (int i, TObj * t = NULL);
  //  Constructor with automatically generated new name

  ~OVar ();
  //  Destructor.

  koObject *Copy () const;
  // Virtual Copy Constructor
    
  //  OResult* reduce ();
  
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "OVAR: " + name

  const char* Name () const {return namestring;}
  // Selector

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 
 
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  NonType* partial_transform(CState* s);
  // similar to transform except one thing.
  // If no match is found for variable,
  // do not report it as an error. Instead,
  // just return variable itself.

  CState* directStore(CState* a);

  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  void initCurrChild() {}
  // This is a dummy function for all the classes 
  // that don't have any child.

  NonType* nextChild() { return NULL; }
  // This is dummy functions for all the classes 
  // that don't have any child.
  // This dummy function returns NULL since there is no next child for 
  // those classes (classes w/o any child.) 

  int deepeq(NonType * other) { return 0; }
  // Value-based equality

  int numChildren() const { return 0; }
  //  Number of arguments.

  char *sql ();
  // Return the SQL string equivalent of this object

  char *oql ();
  // Return the OQL string equivalent of this object

  static int cnt;
  // Used to generate fresh variable names.  Must
  // be initialized with initstats ()

  char *Val ();
  // Selector

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject *object() { return this; }
  // Returns the kola equivalent of an object.

  OSObject *result() { return NULL; }
  // Returns the OSObject equivalent of itself.
#endif // _OSTORE_

protected:
  char* val;

};

void initstats ();

#endif

