/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)			
// FILE: KOPInv.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOPInv.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PInv
// Parameters:	(Predicate* eq)				 
//
//  Constructor.  eq is predicate argument.
//         								    
//////////////////////////////////////////////////////////////////////////////

PInv::PInv (Predicate* eq, TPred * t) : Predicate (t, PRED_INV,"INV")
{
    one = eq;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PInv
// Parameters:	(PInv& other)				   
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

PInv::PInv (const PInv& other) : Predicate ((Predicate &) other)
{
    one = other . One () -> Copy ();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~PInv
// Parameters:	()
//
//  Destructor
//         								    
//////////////////////////////////////////////////////////////////////////////

PInv::~PInv ()
{
    delete one;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()
//
//  Return copy of *this as Predicate
//         								    
//////////////////////////////////////////////////////////////////////////////

Predicate* PInv::Copy () const
{
    return new PInv (*this);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:deepeq
// Parameters:(Nontype*)
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////


int PInv::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == PNAME ) && (name_->pn == PRED_INV))
  {
    delete name_;
    return (one->deepeq(((PInv*)other)->One()));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* PInv::Myname()
{
	NameUnion* a = new NameUnion(PNAME, PRED_INV);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//
//   Function Name: nextChild
//   Parameters: ()
// 
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////
NonType* PInv::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	(OResult* a)						    
//
//  return
//    one -> reduce (new OResPAIR (a -> one -> reduce,
//  			       a -> two -> reduce))
//         								    
//////////////////////////////////////////////////////////////////////////////

// BResult* PInv::reduce (OResult* a)
// {

//  if (! (a -> isPair ())) {

//     char *or = one -> representation ();
//     char *r = new char [strlen (or) + strlen ("PINV()") + 1];
//     sprintf (r, "PINV(%s)", or);
//     BResult *res = new BResBB (r, a -> Copy ());
//     delete [] or;
//     delete [] r;
//     delete a;
//     return res;
    
//  } else {
    
//     OResPAIR *a2 = ((OResPAIR* ) a);
//     BResult *res = one -> reduce (new OResPAIR (a2 -> Two () -> Copy (), a2 -> One () -> Copy ()));
//     delete a;
//     return res;
//   }
// }



// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "PFO: INV" + one -> representation ()
//         								    
//////////////////////////////////////////////////////////////////////////////

char* PInv::representation ()
{

    char *pr = one -> representation ();
    char *res = new char [strlen (pr) + strlen ("() \\INV") + 1];
    sprintf (res, "(%s) \\INV", pr);
    delete [] pr;
    return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* PInv::transform(CState* s)
{
  Predicate* first = (Predicate*) One() -> transform(s);
  return new PInv(first ,Type() );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transformDeep
// Parameters:	(CState*, CIEnvStackDataType*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.
//
//////////////////////////////////////////////////////////////////////////////

NonType* PInv::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  Predicate* first = (Predicate*) One() -> transformDeep(s, startingPt);
  return new PInv(first  ,Type()  );
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int PInv::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_INV))
	{
	  // do it for first child.
	  if (one->compares(((PInv*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((PVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((PVar*)one)->Name(), new CIdent((PInv*)nt,((PInv*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((PVar*)one)->Name()))->Child_Look_Up())->deepeq(((PInv*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
} 

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This match method is used for matching statement.
// No duplicated variable entry is allowed.
//
//////////////////////////////////////////////////////////////////////////////

int PInv::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_INV))
	{
	  // do it for first child.
	  if (one->comparesSTM(((PInv*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((PVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((PVar*)one)->Name(), new CIdent((PInv*)nt,((PInv*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else
				{
				  return_val++;
				  cout << "Error : Variable " << ((PVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
  
} 

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list and inverse dependency list
//
//////////////////////////////////////////////////////////////////////////////

void PInv::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on inverse dependency list.
// undo for dependency list is not necessary, since 
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void PInv::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void PInv::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: invoke
//  Parameters:(koObject*)
//
//  Implements the inverse predicate former for koObjects.
//
//////////////////////////////////////////////////////////////////////////////

koBool* PInv::invoke(koObject* x)
{
  return new BInvoke(this,x);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Implements the inverse predicate former for OSObjects.
//
//////////////////////////////////////////////////////////////////////////////

OSBool* PInv::exec(OSObject* x)
{
  return One()->exec(new OSPair(((OSPair*)x)->Two(),((OSPair*)x)->One()));
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* PInv::prolog ()
{
    char *pr = one -> prolog ();
    char *res = new char [strlen (pr) + strlen ("pinv()") + 1];
    sprintf (res, "pinv(%s)", pr);
    delete [] pr;
    return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref PInv::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("pinv"),1,or);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  inv
// Parameters:	(koObject *o)
//
// Symbolic invocation of this on o.  Eventually,
// this method will replace "invoke" presently
// only defined with _OSTORE_ flag set
//
/////////////////////////////////////////////////////////////////////////////

koBool *PInv::inv (koObject *o) 
{
  NameUnion *nu = o -> Myname ();

  if (nu ->on == OBJ_PAIR) {
	delete nu;
	OPair *op = new OPair (o -> asPair () -> Two () -> Copy (), o -> asPair () -> One () -> Copy ());
	koBool *res = one -> inv (op);
	delete op;
	return res;
  } else {
	delete nu;
	cerr << "Invoking PInv on Non-Pair Object" << endl;
    return NULL;
  }
}
