/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOPIsnull.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOPIsnull.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PIsnull
// Parameters:	()							  
//
//  Constructor. 
//         								    
//////////////////////////////////////////////////////////////////////////////

PIsnull::PIsnull (TPred * t)  : Predicate (t, PRED_ISNULL,"ISNULL")
{    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PIsnull
// Parameters:	(PIsnull& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

PIsnull::PIsnull (const PIsnull& other) : Predicate ((Predicate &) other)
{
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~PIsnull
// Parameters:	()
//
//  Destructor
//         								    
//////////////////////////////////////////////////////////////////////////////

PIsnull::~PIsnull ()
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()
//
//  Return copy of *this as Predicate
//         								    
//////////////////////////////////////////////////////////////////////////////

Predicate* PIsnull::Copy () const
{
   return new PIsnull (*this);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:deepeq
// Parameters:(Nontype*)
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////

int PIsnull::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == PNAME ) && (name_->pn == PRED_ISNULL))
  {
    delete name_;
    return T;
  }
  else
  {
    delete name_;
    return F;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* PIsnull::Myname()
{
	NameUnion* a = new NameUnion(PNAME, PRED_ISNULL);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "PFO: ISNULL" + one -> representation ()
//         								    
//////////////////////////////////////////////////////////////////////////////

char* PIsnull::representation ()
{
  char *res = new char [strlen ("ISNULL") + 1];
  sprintf (res, "%s", "ISNULL");
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation 
// Parameters:	()							  
//  return new BResINF (a -> fst -> reduce (), "=", 
//  		    a -> snd -> reduce ())
//
/////////////////////////////////////////////////////////////////////////////

// BResult* PIsnull::reduce (OResult* a)
// {
//     BResBB *res = new BResBB ("ISNULL", a -> Copy ());
//     delete a;
//     return res;
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding Nontype object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* PIsnull::transform(CState* s)
{
   
    return new PIsnull(Type());
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int PIsnull::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_ISNULL))
	{
	  delete tmp_name;
	  return T;
	}
  else 
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void PIsnull::replaceChild(NonType* child, int which_child)
{

}
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* PIsnull::prolog ()
{
  char *res = new char [strlen ("pisnull") + 1];
  sprintf (res, "pisnull");
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref PIsnull::sp_prolog ()
{
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(res,"pisnull");
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PIsnull::sql (koObject *o)
{    
    char *arg = o -> sql ();
    if (strncmp (arg, "ERROR", 5) == 0)
      return arg;
    else {
      char *res = new char [strlen ("( IS NULL)") + strlen (arg) + 1];
      sprintf (res, "(%s IS NULL)", arg);
      delete [] arg;
	  return res;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PIsnull::oql (koObject *o)
{    
    char *arg = o -> oql ();
    if (strncmp (arg, "ERROR", 5) == 0)
      return arg;
    else {
      char *res = new char [strlen ("( IS NULL)") + strlen (arg) + 1];
      sprintf (res, "(%s IS NULL)", arg);
      delete [] arg;
	  return res;
    }
}

