/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	 
// FILE: TYTUser.C
// PROJ: KOLA Parser, AQUA Translator
// DATE: 5/95
//
//////////////////////////////////////////////////////////////////////////////

#include "TYTUser.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TUser
// Parameters:	(char* n)							  
//
//  Constructor.  n is the name associated with this type
//         								    
//////////////////////////////////////////////////////////////////////////////

TUser::TUser (char* n)
{

    name = new char [strlen (n) + 1];
    sprintf (name, "%s", n);
    myname = TYPE_USER;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TUser
// Parameters:	(char* n, char *n2)							  
//
//  Constructor.  n + n2 is the name associated with this type
//         								    
//////////////////////////////////////////////////////////////////////////////

TUser::TUser (char* n, char *n2)
{

    name = new char [strlen (n) + strlen (n2) + strlen (" ") + 1];
    sprintf (name, "%s %s", n, n2);
    myname = TYPE_USER;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TUser
// Parameters:	(TUser& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

TUser::TUser (TUser& other)
{

    name = new char [strlen (other . Name ()) + 1];
    sprintf (name, "%s", other . Name ());
    myname = TYPE_USER;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~TUser
// Parameters:	()
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

TUser::~TUser ()
{    
  if (name) { delete [] name; name = NULL; }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  return "VARIABLE: " + name
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TUser::representation ()
{    
    char *res = new char [strlen (name) + 1];
    sprintf (res, "%s", name);
    return res;    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  prolog
// Parameters:	()							  
//
//  return "VARIABLE: " + name
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TUser::prolog ()
{    
    char *res = new char [strlen ("tuser()") + strlen (name) + 2];
    sprintf (res, "tuser(k%s)", name);
    return res;    
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref TUser::sp_prolog ()
{
  char *text = new char [strlen (name) + 2];
  sprintf (text, "k%s", name);

  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("tuser"),1,text);

  if (text) { delete [] text; text = NULL; }

  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as TObj*
//         								    
//////////////////////////////////////////////////////////////////////////////

TObj* TUser::Copy ()
{

    return new TUser (*this);

}

